using namespace std;


//get the sum of the phenotype observations
//the input flist can be any group of the phenotype observations
float get_T(const qtPhenotype& flist)
{
	qtPhenotype::const_iterator iter;

	float T=0;

	for (iter=flist.begin();iter!=flist.end();++iter)
	{
		T = T + *iter;
	}

	return T;
}

//get the sum of the phenotype observations
//the input flist can be any group of the phenotype observations
float get_T(const list<float>& flist)
{
	list<float>::const_iterator iter;

	float T=0;

	for (iter=flist.begin();iter!=flist.end();++iter)
	{
		T = T + *iter;
	}

	return T;
}


//get the correction for the mean
//the input is the phenotype Y
//only need to be called once in the entire program
float get_CM(const qtPhenotype& Y)
{
	float CM=0;

	float T=0;

	T = get_T(Y);

	CM = (T*T)/TOTAL_NUMBER_OF_INDIVIDUALS;

	return CM;

}


//get the total sum of squares for the entire phenotye observations
//the input is the phenotype Y
//only need to be called once in the entire program
float get_SS_Total(const qtPhenotype& Y, const float& CM)
{
	float SST = 0;

	qtPhenotype::const_iterator iter;

	for (iter=Y.begin();iter!=Y.end();++iter)
	{
		SST = SST + (*iter)*(*iter);
	}

	SST = SST - CM;

	return SST;
}


//get the sum of squares between the groups for ctable_2
float get_SS_Between_ctable_2(const list<float> ctable_2[2], const float& CM)
{
	float SSB=0;

	float T=0;

	for (int i=0;i<2;i++)
	{
		if (ctable_2[i].size()!=0) 
		{
			T = get_T(ctable_2[i]);

			SSB = SSB + ((T*T)/ctable_2[i].size());
		}

	}

	SSB = SSB - CM;
	
	return SSB;
}


//get the sum of squares between the groups for ctable_4
float get_SS_Between_ctable_4(const list<float> ctable_4[4], const float& CM)
{
	float SSB=0;

	float T=0;

	for (int i=0;i<4;i++)
	{
		if (ctable_4[i].size()!=0) 
		{
			T = get_T(ctable_4[i]);

			SSB = SSB + ((T*T)/ctable_4[i].size());
		}

	}

	SSB = SSB - CM;
	
	return SSB;
}


//get the F test of ctable_2
float get_F_ctable_2(const list<float> ctable_2[2], const float& SS_Total, const float& CM)
{
	float F=0;

	float SSB = 0;

	SSB = get_SS_Between_ctable_2(ctable_2, CM);

	if ((SS_Total - SSB)!=0) 
	{
		F = (TOTAL_NUMBER_OF_INDIVIDUALS-2)*(SSB)/(SS_Total - SSB);
	}

	else cout<<"SS_Total -SSB == 0 !"<<endl;

	return F;
}


//get the F test of ctable_4
float get_F_ctable_4(const list<float> ctable_4[4], const float& SS_Total, const float& CM)
{
	float F=0;

	float SSB = 0;

	SSB = get_SS_Between_ctable_4(ctable_4, CM);

	int df=4;

	for (int i=0;i<4;i++)
	{
		if (ctable_4[i].size()==0)
		{
			df =3;

			break;

		}

	}

	if ((SS_Total - SSB)!=0) 
	{
		if (df == 3) 
		{
			F = ((TOTAL_NUMBER_OF_INDIVIDUALS-2)*(SSB)/(SS_Total - SSB))/2;
		}

		else	F = ((TOTAL_NUMBER_OF_INDIVIDUALS-4)*(SSB)/(SS_Total - SSB))/3;
	}


	else cout<<"SS_Total -SSB == 0 !"<<endl;


	return F;

}

float get_F(const qtPhenotype& Y, const biSNP& X1, const biSNP& X2, const float& SS_Total, const float& CM)
{
    qtPhenotype::const_iterator iter;

	int i=TOTAL_NUMBER_OF_INDIVIDUALS-1;

	float T[4]={0};
	int count[4]={0};
	
	
	for (iter=Y.begin();iter!=Y.end();++iter)
	{
	    if (X1.test(i)==0) 
	    {
		if (X2.test(i)==0){
		    T[0]=T[0]+ *iter;
		    count[0]++;		    
		    //ctable_4[0].push_back(*iter);
		}
		
		else{		    
		    T[1]=T[1]+ *iter;
		    count[1]++;		    
		    //ctable_4[1].push_back(*iter);
		}		
	    }
	    else
	    {
		if (X2.test(i)==0) {		    
		    T[2]=T[2]+ *iter;
		    count[2]++;		    
		    //ctable_4[2].push_back(*iter);
		}		
		else{
		    T[3]=T[3]+ *iter;
		    count[3]++;		    
		    //ctable_4[3].push_back(*iter);
		}
		
	    }
	    i--;
	}
	
	float F=0;

	float SSB = 0;
	int df=4;
	
	for (int i=0;i<4;i++)
	{
	    if (count[i]>0)
		SSB = SSB + ((T[i]*T[i])/count[i]);	    
	    else
		df=3;
	    
	}
	SSB = SSB - CM;
	
	//SSB = get_SS_Between_ctable_4(ctable_4, CM);

	if ((SS_Total - SSB)!=0) 
	{
	    if (df == 3) 
	    {
		F = ((TOTAL_NUMBER_OF_INDIVIDUALS-2)*(SSB)/(SS_Total - SSB))/2;
	    }

	    else	F = ((TOTAL_NUMBER_OF_INDIVIDUALS-4)*(SSB)/(SS_Total - SSB))/3;
	}


	else cout<<"SS_Total -SSB == 0 !"<<endl;
	return F;

}
